#include <Wire.h>
#include "MAX30100_PulseOximeter.h"
#include <LiquidCrystal_I2C.h>
#include <SHT21.h>

#define REPORTING_PERIOD_MS     3000
#define CHOICE_PERIOD 10
#define PULSE_THRESHOLD 70
#define HUMIDITY_THRESHOLD 80


#define LED 13
#define BUZZER 6
#define RED_LED 11
#define GREEN_LED 10
#define BLUE_LED 9


LiquidCrystal_I2C lcd(0x27, 16, 2);
PulseOximeter pox;
SHT21 sht;


float pulse_sum = 0;
float pulse_avg = 0;

float humidity_sum = 0;
float humidity_avg = 0;

int is_lying = -1;
int count = 0;

float Heart_rate;
float Humidity;

uint32_t tsLastReport = 0;

#define NOTE_B0  31
#define NOTE_C1  33
#define NOTE_CS1 35
#define NOTE_D1  37
#define NOTE_DS1 39
#define NOTE_E1  41
#define NOTE_F1  44
#define NOTE_FS1 46
#define NOTE_G1  49
#define NOTE_GS1 52
#define NOTE_A1  55
#define NOTE_AS1 58
#define NOTE_B1  62
#define NOTE_C2  65
#define NOTE_CS2 69
#define NOTE_D2  73
#define NOTE_DS2 78
#define NOTE_E2  82
#define NOTE_F2  87
#define NOTE_FS2 93
#define NOTE_G2  98
#define NOTE_GS2 104
#define NOTE_A2  110
#define NOTE_AS2 117
#define NOTE_B2  123
#define NOTE_C3  131
#define NOTE_CS3 139
#define NOTE_D3  147
#define NOTE_DS3 156
#define NOTE_E3  165
#define NOTE_F3  175
#define NOTE_FS3 185
#define NOTE_G3  196
#define NOTE_GS3 208
#define NOTE_A3  220
#define NOTE_AS3 233
#define NOTE_B3  247
#define NOTE_C4  262
#define NOTE_CS4 277
#define NOTE_D4  294
#define NOTE_DS4 311
#define NOTE_E4  330
#define NOTE_F4  349
#define NOTE_FS4 370
#define NOTE_G4  392
#define NOTE_GS4 415
#define NOTE_A4  440
#define NOTE_AS4 466
#define NOTE_B4  494
#define NOTE_C5  523
#define NOTE_CS5 554
#define NOTE_D5  587
#define NOTE_DS5 622
#define NOTE_E5  659
#define NOTE_F5  698
#define NOTE_FS5 740
#define NOTE_G5  784
#define NOTE_GS5 831
#define NOTE_A5  880
#define NOTE_AS5 932
#define NOTE_B5  988
#define NOTE_C6  1047
#define NOTE_CS6 1109
#define NOTE_D6  1175
#define NOTE_DS6 1245
#define NOTE_E6  1319
#define NOTE_F6  1397
#define NOTE_FS6 1480
#define NOTE_G6  1568
#define NOTE_GS6 1661
#define NOTE_A6  1760
#define NOTE_AS6 1865
#define NOTE_B6  1976
#define NOTE_C7  2093
#define NOTE_CS7 2217
#define NOTE_D7  2349
#define NOTE_DS7 2489
#define NOTE_E7  2637
#define NOTE_F7  2794
#define NOTE_FS7 2960
#define NOTE_G7  3136
#define NOTE_GS7 3322
#define NOTE_A7  3520
#define NOTE_AS7 3729
#define NOTE_B7  3951
#define NOTE_C8  4186
#define NOTE_CS8 4435
#define NOTE_D8  4699
#define NOTE_DS8 4978
#define REST 0

int lying_melody[] = {

  // Never Gonna Give You Up - Rick Astley
  // Score available at https://musescore.com/chlorondria_5/never-gonna-give-you-up_alto-sax
  // Arranged by Chlorondria

  NOTE_D5,-4, NOTE_E5,-4, NOTE_A4,4, //1
  NOTE_E5,-4, NOTE_FS5,-4, NOTE_A5,16, NOTE_G5,16, NOTE_FS5,8,
  NOTE_D5,-4, NOTE_E5,-4, NOTE_A4,2,
  NOTE_A4,16, NOTE_A4,16, NOTE_B4,16, NOTE_D5,8, NOTE_D5,16,
  NOTE_D5,-4, NOTE_E5,-4, NOTE_A4,4, //repeat from 1
  NOTE_E5,-4, NOTE_FS5,-4, NOTE_A5,16, NOTE_G5,16, NOTE_FS5,8,
  NOTE_D5,-4, NOTE_E5,-4, NOTE_A4,2,
  NOTE_A4,16, NOTE_A4,16, NOTE_B4,16, NOTE_D5,8, NOTE_D5,16,
  REST,4, NOTE_B4,8, NOTE_CS5,8, NOTE_D5,8, NOTE_D5,8, NOTE_E5,8, NOTE_CS5,-8,
  NOTE_B4,16, NOTE_A4,2, REST,4, 

  REST,8, NOTE_B4,8, NOTE_B4,8, NOTE_CS5,8, NOTE_D5,8, NOTE_B4,4, NOTE_A4,8, //7
  NOTE_A5,8, REST,8, NOTE_A5,8, NOTE_E5,-4, REST,4, 
  NOTE_B4,8, NOTE_B4,8, NOTE_CS5,8, NOTE_D5,8, NOTE_B4,8, NOTE_D5,8, NOTE_E5,8, REST,8,
  REST,8, NOTE_CS5,8, NOTE_B4,8, NOTE_A4,-4, REST,4,
  REST,8, NOTE_B4,8, NOTE_B4,8, NOTE_CS5,8, NOTE_D5,8, NOTE_B4,8, NOTE_A4,4,
  NOTE_E5,8, NOTE_E5,8, NOTE_E5,8, NOTE_FS5,8, NOTE_E5,4, REST,4,
   
  NOTE_D5,2, NOTE_E5,8, NOTE_FS5,8, NOTE_D5,8, //13
  NOTE_E5,8, NOTE_E5,8, NOTE_E5,8, NOTE_FS5,8, NOTE_E5,4, NOTE_A4,4,
  REST,2, NOTE_B4,8, NOTE_CS5,8, NOTE_D5,8, NOTE_B4,8,
  REST,8, NOTE_E5,8, NOTE_FS5,8, NOTE_E5,-4, NOTE_A4,16, NOTE_B4,16, NOTE_D5,16, NOTE_B4,16,
  NOTE_FS5,-8, NOTE_FS5,-8, NOTE_E5,-4, NOTE_A4,16, NOTE_B4,16, NOTE_D5,16, NOTE_B4,16,

};

int lying_tempo = 114;

int lying_notes = sizeof(lying_melody) / sizeof(lying_melody[0]) / 2;

int lying_wholenote = (60000 * 4) / lying_tempo;

int lying_divider = 0, lying_noteDuration = 0;



int truth_melody[] = {


  // Hedwig's theme fromn the Harry Potter Movies
  // Socre from https://musescore.com/user/3811306/scores/4906610
  
  REST, 2, NOTE_D4, 4,
  NOTE_G4, -4, NOTE_AS4, 8, NOTE_A4, 4,
  NOTE_G4, 2, NOTE_D5, 4,
  NOTE_C5, -2, 
  NOTE_A4, -2,
  NOTE_G4, -4, NOTE_AS4, 8, NOTE_A4, 4,
  NOTE_F4, 2, NOTE_GS4, 4,
  NOTE_D4, -1, 
  NOTE_D4, 4,

  NOTE_G4, -4, NOTE_AS4, 8, NOTE_A4, 4, //10
  NOTE_G4, 2, NOTE_D5, 4,
  NOTE_F5, 2, NOTE_E5, 4,
  NOTE_DS5, 2, NOTE_B4, 4,
  NOTE_DS5, -4, NOTE_D5, 8, NOTE_CS5, 4,
  NOTE_CS4, 2, NOTE_B4, 4,
  NOTE_G4, -1,
  NOTE_AS4, 4 
  
};


int truth_notes = sizeof(truth_melody) / sizeof(truth_melody[0]) / 2;

int truth_tempo = 144;

int truth_wholenote = (60000 * 4) / truth_tempo;

int truth_divider = 0, truth_noteDuration = 0;


 
void onBeatDetected()
{
    tone(BUZZER, 1000, 100);
    digitalWrite(LED,!digitalRead(LED));
}



void setup()
{
    Serial.begin(115200);
    lcd.begin(16,2);
    
    // Turn on the blacklight and print a message.
    lcd.backlight();
    
    lcd.setCursor(0,0);
    lcd.print("Initializing...");
    delay(1000);
    lcd.setCursor(0,1);
    if (!pox.begin()) {
        lcd.print("FAILED");
        while(1);
    } else {
        lcd.print("SUCCESS");
    }
    pinMode(LED, OUTPUT);
    pinMode(BUZZER, OUTPUT);
    pinMode(RED_LED, OUTPUT);
    pinMode(GREEN_LED, OUTPUT);
    pinMode(BLUE_LED, OUTPUT);
    
    pox.setOnBeatDetectedCallback(onBeatDetected);
}
 
void loop()
{
    // Make sure to call update as fast as possible
    pox.update();
    RGB_color(0, 0, 0);
    if (millis() - tsLastReport > REPORTING_PERIOD_MS) {
      
        Heart_rate = pox.getHeartRate();
        Serial.print("Heart rate:");
        Serial.println(Heart_rate);
        
        // to 16x2 LCD
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("HR: ");
        lcd.print(pox.getHeartRate());
        lcd.print(" bpm");
        
        Humidity = sht.getHumidity();
        erial.print("Humidity:");
        Serial.println(Humidity);

        // to 16x2 LCD
        lcd.setCursor(0,1);
        lcd.print("Humi: ");
        lcd.print(Humidity);
        lcd.print(" %");

        
         if (Heart_rate >= 60) {
          pulse_sum += Heart_rate;
          humidity_sum += Humidity;
          count = count + 1;
        }
        
        tsLastReport = millis();
    }
    make_choice();
    if (is_lying > 0) {
        sing_lying_song();
     } else if (is_lying == 0){
        sing_telling_the_truth_song();
    }
    
}

void make_choice() 
{
    if (count == CHOICE_PERIOD) {
        humidity_avg = humidity_sum / CHOICE_PERIOD;
        pulse_avg = pulse_sum / CHOICE_PERIOD;
        Serial.println(humidity_avg);
        Serial.println(pulse_avg);

        if (humidity_avg > HUMIDITY_THRESHOLD && pulse_avg > PULSE_THRESHOLD) {
            is_lying = 1;
        } else {
           is_lying = 0;
        }
        count = 0;
        humidity_sum = 0;
        pulse_sum = 0;
    }
}
void sing_lying_song()
{
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("You are lying!");
    RGB_color(255, 0, 0);
    for (int thisNote = 0; thisNote < lying_notes * 2; thisNote = thisNote + 2) {
      // calculates the duration of each note
      
      lying_divider = lying_melody[thisNote + 1];
      if (lying_divider > 0) {
        // regular note, just proceed
        lying_noteDuration = (lying_wholenote) / lying_divider;
      } else if (lying_divider < 0) {
        // dotted notes are represented with negative durations!!
        lying_noteDuration = (lying_wholenote) / abs(lying_divider);
        lying_noteDuration *= 1.5; // increases the duration in half for dotted notes
      }
  
      // we only play the note for 90% of the duration, leaving 10% as a pause
      tone(BUZZER, lying_melody[thisNote], lying_noteDuration*0.9);
  
      // Wait for the specief duration before playing the next note.
      delay(lying_noteDuration);
  
      // stop the waveform generation before the next note.
      noTone(BUZZER);
      
    }
    is_lying = -1;
    tsLastReport = 0;
    pox.begin();  
}

void sing_telling_the_truth_song() 
{
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("You are telling ");
    lcd.setCursor(0,1);
    lcd.print("  the truth!");
    RGB_color(0, 255, 0);
    for (int thisNote = 0; thisNote < truth_notes * 2; thisNote = thisNote + 2) {
      // calculates the duration of each note
      
      truth_divider = truth_melody[thisNote + 1];
      if (truth_divider > 0) {
        // regular note, just proceed
        truth_noteDuration = (truth_wholenote) / truth_divider;
      } else if (truth_divider < 0) {
        // dotted notes are represented with negative durations!!
        truth_noteDuration = (truth_wholenote) / abs(truth_divider);
        truth_noteDuration *= 1.5; // increases the duration in half for dotted notes
      }

      // we only play the note for 90% of the duration, leaving 10% as a pause
      tone(BUZZER, truth_melody[thisNote], truth_noteDuration*0.9);

      // Wait for the specief duration before playing the next note.
      delay(truth_noteDuration);

      // stop the waveform generation before the next note.
      noTone(BUZZER);
      
    }
    is_lying = -1;
    tsLastReport = 0;
    pox.begin(); 
}
void RGB_color(int red_light_value, int green_light_value, int blue_light_value)
{
    analogWrite(RED_LED, red_light_value);
    analogWrite(GREEN_LED, green_light_value);
    analogWrite(BLUE_LED, blue_light_value);
}
